import java.io.OutputStream;
import java.io.IOException;

/**
 * PakattavaTiedosto on pakattavan tiedoston kirjoittamiseen tarkoitettu luokka, jonka
 * avulla tiedostosta saa luettua halutun mittaisia (bittein) koodeja. Se toimii siis
 * ernlaisena pienen puskurina vliss.
 */
public class PakattavaTiedosto {
	private int maxKoodipituus;				// suurin tiedostossa kytettv koodipituus
	private int puskuri;						// puskurissa silytetn viel lukemattomia
	private int puskurinKoko;					//   bittej
	private boolean saaKirjoittaa;				// saako tiedostoon viel kirjoittaa
	private int tavujaKirjoitettu;				// montako tavua on kirjoitettu tiedostoon
	private OutputStream tiedosto;				// kirjoitettava tiedosto
	
	/**
	 * Luo uuden pakatun tiedoston, joka kytt annettua maksimikoodipituutta.
	 * @param maxKoodipituus	Suurin sallittu koodipituus, jolla tiedostoon kirjoitetaan.
	 * @param tiedosto			Itse pakattava tiedosto.
	 */
	public PakattavaTiedosto(int maxKoodipituus, OutputStream tiedosto) throws IOException {
		this.maxKoodipituus = maxKoodipituus;
		this.tiedosto = tiedosto;
		this.puskuri = 0;
		this.puskurinKoko = 0;
		this.saaKirjoittaa = true;
		this.tiedosto.write(this.maxKoodipituus);
		this.tavujaKirjoitettu = 1;
	}
	
	/**
	 * Kirjoittaa koodin annetulla koodipituudella tiedostoon.
	 * @param koodi			Tiedostoon kirjoitettava koodi.
	 * @param koodipituus		Monellako bitill koodi kirjoitetaan.
	 */
	public void kirjoita(int koodi, int koodipituus) throws IOException {
		if( !this.saaKirjoittaa )				// eik saa kirjoittaa?
			throw new IOException("Tiedosto on ptetty eik siihen voi en kirjoittaa.");
		
		// listn koodi puskuriin
		this.puskuri <<= koodipituus;
		this.puskuri |= koodi;
		this.puskurinKoko += koodipituus;
		
		// kirjoitetaan puskurista tiedostoon niin paljon kuin mahdollista
		while( this.puskurinKoko >= 8 ) {
			this.puskurinKoko -= 8;
			this.tiedosto.write(this.puskuri >>> this.puskurinKoko);
			++this.tavujaKirjoitettu;
			this.puskuri &= (1 << this.puskurinKoko) - 1;		// nollataan kirjoitettu tavu
		}
	}
	
	/**
	 * Ptt kirjoitettavan tiedoston, eli kirjoittaa loputkin puskurista tiedostoon.
	 */
	public void paata() throws IOException {
		if( this.puskurinKoko > 0 ) {
			this.tiedosto.write(this.puskuri << (8 - this.puskurinKoko));
			++this.tavujaKirjoitettu;
		}
		this.saaKirjoittaa = false;			// pttmisen jlkeen tiedostoon ei saa kirjoittaa
	}
	
	/**
	 * Kertoo thn menness kirjoitettujen tavujen mrn.
	 * @return					Kirjoitetun datan mrn tavuissa.
	 */
	public int paljonkoKirjoitettu() {
		return this.tavujaKirjoitettu;
	}
}
